/**
SyntaxHighlighter brush for ABNF grammars
=========================================
This repository provides a SyntaxHighlighter brush for W3C's ABNF grammar format.

License
=======
Copyright (c) 2010 Nu Echo Inc

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
**/

SyntaxHighlighter.brushes.Abnf = function() {
   var keywords = 'private public root language meta is lexicon base tag-format mode';
   this.regexList = [
      { regex: new RegExp('^#ABNF .*$', 'gmi'),                           css: 'directive' },
      { regex: SyntaxHighlighter.regexLib.singleLineCComments,	          css: 'comments' },		// one line comments
      { regex: /\/\*([^\*][\s\S]*)?\*\//gm,				  css: 'comments' },	 	// multiline comments
      { regex: /\/\*(?!\*\/)\*[\s\S]*?\*\//gm,				  css: 'preprocessor' },	// documentation comments
      { regex: SyntaxHighlighter.regexLib.multiLineDoubleQuotedString,    css: 'qstring' },
      { regex: new RegExp('{!{(.|\\n)*}!}', 'gmi'),                       css: 'action' },
      { regex: new RegExp('{[^}]+}', 'gmi'),                              css: 'action' },
      { regex: new RegExp('\\$[^ \\n\\t]+', 'gmi'),                       css: 'rulename' },
      { regex: new RegExp('@[a-z]+', 'gmi'),                              css: 'directive' },
      { regex: new RegExp(this.getKeywords(keywords), 'gmi'),             css: 'keyword' }
      ];
};

SyntaxHighlighter.brushes.Abnf.prototype = new SyntaxHighlighter.Highlighter();
SyntaxHighlighter.brushes.Abnf.aliases  = ['abnf'];

